import 'package:flutter/material.dart';
import 'package:flutter_custom_toolbox/flutter_toolbox.dart';

import 'package:tetrisdual/config/default_game_settings.dart';
import 'package:tetrisdual/models/settings/settings_game.dart';

part 'settings_game_state.dart';

class GameSettingsCubit extends HydratedCubit<GameSettingsState> {
  GameSettingsCubit() : super(GameSettingsState(settings: GameSettings.createDefault()));

  void setValues({
    String? gameType,
  }) {
    emit(
      GameSettingsState(
        settings: GameSettings(
          gameType: gameType ?? state.settings.gameType,
        ),
      ),
    );
  }

  String getParameterValue(String code) {
    switch (code) {
      case DefaultGameSettings.parameterCodeGameType:
        return GameSettings.getGameTypeValueFromUnsafe(state.settings.gameType);
    }

    return '';
  }

  void setParameterValue(String code, String value) {
    final String gameType = code == DefaultGameSettings.parameterCodeGameType
        ? value
        : getParameterValue(DefaultGameSettings.parameterCodeGameType);

    setValues(
      gameType: gameType,
    );
  }

  @override
  GameSettingsState? fromJson(Map<String, dynamic> json) {
    final String gameType = json[DefaultGameSettings.parameterCodeGameType] as String;

    return GameSettingsState(
      settings: GameSettings(
        gameType: gameType,
      ),
    );
  }

  @override
  Map<String, dynamic>? toJson(GameSettingsState state) {
    return <String, dynamic>{
      DefaultGameSettings.parameterCodeGameType: state.settings.gameType,
    };
  }
}
